# $Id: customerConfig.py 3042 2010-02-09 08:54:39Z anton $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-8.3.0/src/customerConfig.py $

import os
import ConfigParser
from os.path import isfile
import sys, traceback
import re

class CustomerConfig:
      """
      Provide simple configuration file support, for multiple languages (default: 'en' i.e. English).
      The code uses the standard python configparser:  http://docs.python.org/library/configparser.html
      The format of the configfile is thus presribed by the configparser.

      The most important call is:

      get(option, section, default)

      The value returned is:
        1 The value actually found. If not present:
        2 The value of the 'default' parameter. If not present:
        3 The content of the 'option' parameter. If not present:
        4 None, raise CustomerConfigException
      """
      def __init__(self, configFile, infra, language = 'en', section = ''):
            self.infra      = infra
            self.logging    = infra.getLoggingInstance()
            self.logger     = self.logging.getLogger('meresco.configs')
            self.configFile = configFile
            #FIXME Enlish rules!
            self.language   = 'en' #language
            self.section    = section
            self.configParser     = ConfigParser.ConfigParser()

            # allow options to be case sensitive!
#            self.configParser.optionxform = str

            if not isfile(configFile):
               self.logger.error("Configfile NOT FOUND: " + configFile)
            else:
               self.configParser.read(configFile)
            self.logger.debug('Init for customerConfig, file: '+ configFile)

      def setDefaults(self, section, language = 'en'):
            self.section = section
            self.language= 'en'

      def setSection(self, section):
            self.section = section

      def getConfig(self):
            return self.configParser

      def setLanguage(self, language):
            self.language = 'en'

      def setSubst(self, substDict):
            self.substDict = substDict

      def crlfList(self, string, skipEmptyLines):
            collection = []
            if string != None:
                  collection =  string.split("\n")
                  if skipEmptyLines:
                        collection = filter(lambda x: len(x), collection)
            return collection

      # returns a list of strings by splitting the variable's "\n" and removing empty lines
      def getList(self, option, default = [], section = '', substDict = {}, skipEmptyLines = True):
            string = self.get( option, section=section, substDict=substDict, default=default )
            return self.crlfList(string, skipEmptyLines)


      # returns a hash of an entire section
      def getSectionDict(self,  section = '', substDict = {}, skipEmptyLines = True):
            hash = {}
            options  = self.configParser.items(section)
            for option,value in options:
                  hash[option] = value
            return hash

      def getSectionDictList(self,  section = '', substDict = {}, skipEmptyLines = True):
            hash = {}
            options  = self.configParser.items(section)
            for option,value in options:
                  hash[option] = self.crlfList(value, skipEmptyLines)
            return hash

      def getSectionTuple(self,  section = '', substDict = {}, skipEmptyLines = True):
            return self.configParser.items(section)

      def get(self, option, default = '', section = '',substDict={}):
            config = self.configParser
            logger = self.logger
            sane = True
            value = ''
            crlf = re.compile('\n')

            if (substDict == ''): substDict=self.substDict
            if (section == ''): section = self.section

            if (option == None or option == ''):
                  logger.error("No valid option specified")
                  sane = False

            if (section == None or section == '') :
                  logger.error("No section specified")
                  sane = False

            if (default != '' and default != None ):
                  value = default
            elif (option != '' and option != None ):
                  value = option

            if (sane):
                  try:
                        value = config.get(section, option, 0, substDict)
                  except:
                        try:
                              value = config.get(section, option, 0, substDict)
                        except Exception, e:
                              stack = traceback.format_stack()
                              stack.pop()
                              el = stack.pop()

                              logger.error ('"' + str(option) + '[_' + self.language + ']'  +
                                              '" NOT specified ' + 'in file: "' +
                                              self.configFile + '" section: "' + str(section) + '". Using default value: "' +
                                              str(default)+ '" At: ' + crlf.sub('', el) )


            else:
                  if (value == ''):
                        value = "*Incomplete Request*"
                        logger.error("CustomerConfig: Incomplete Request. Called from: ")
                        stack.pop()
                        el = stack.pop()
                        logger.warning('>' + crlf.sub('', el))

            if (value):
                  logger.debug('returning value: [' + value +']')

            return value

      def dump(self):
            s = ''
            for section in config.sections():
                  s = s + ("section: " +  section )
                  for option in config.options(section):
                        s = s + " " + option + "=" + config.get(section, option) + "\n"
            return s




